;##################################################################
; COMImageList
;##################################################################
#include "COMImageList.h"

/*
This program requires Common Controls version 6.
Because of the way side by side assembly works you must include a manifest file
with the program (its in the folder). Also the comctl32.dll file found in Windows\System32
is usually version 5.8x and the ImageList_CoCreateInstance will not be found there. Because
GoAsm will import directly from the DLL this means the program will not assemble
correctly. In order to fix this you must find comctl32.dll version 6, in Vista it can be
found here:

C:\Windows\winsxs\x86_microsoft.windows.common-controls_6595b64144ccf1df_6.0.6002.18005_none_5cb72f96088b0de0

Copy the DLL to the project folder and assemble, GoAsm should find the import OK.
*/

DATA SECTION

	// A couple of GUIDs we'll need to instantiate the object
	CLSID_ImageList GUID GUID_CLSID_ImageList
	IID_IImageList2 GUID GUID_IID_IImageList2

	// Handles used in the example
	hInstance	HANDLE ?
	hBmp		HANDLE ?
	hToolBar	HANDLE ?

	// Pointer to the image list objects, doubles as handles
	pImagelist	PTR ?
	pImageListLarge	PTR ?
	
	pBitmap	PTR ?

	// Used to return information from the IImageList2 calls
	returndata	DD ?

	// An array of TBBUTTON structures used to initialize the toolbar
	ALIGN 16
	tbrbtns				TBBUTTON	<1,1000,TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0,0>
						TBBUTTON	<0,1001,TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0,0>
						TBBUTTON	<2,1002,TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0,0>
						TBBUTTON	<3,1003,TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0,0>
						TBBUTTON	<4,1004,TBSTATE_ENABLED,TBSTYLE_BUTTON,0,0,0>
	tbbend:
	nToolbarBtns		equ	(tbbend-tbrbtns) /SIZEOF TBBUTTON

	// This is used to get the height of a button, used to initialize the image list
	bmpinfo				BITMAP		<>

CODE SECTION

START:
	invoke GetModuleHandle, 0
	mov [hInstance],eax
	// COM must be initialized
	invoke CoInitialize,NULL
	// Turn on common controls
	invoke InitCommonControls
	// Display the dialog
	invoke DialogBoxParam,[hInstance],1000,0,ADDR DlgProc,0
	// Destroy the image lists and release the object
	CoInvoke(pImagelist,IImageList2.Release)
	CoInvoke(pImageListLarge,IImageList2.Release)
	// Turn COM off
	invoke CoUninitialize
	invoke ExitProcess,0

DlgProc FRAME hwnd,uMsg,wParam,lParam

	cmp D[uMsg],WM_INITDIALOG
	jne >>.WM_COMMAND
		// Create an instance of the IImageList2 object
		invoke ImageList_CoCreateInstance,offset CLSID_ImageList,NULL,offset IID_IImageList2, offset pImagelist

		// We'll just recycle a toolbar from another application
		invoke LoadImage,NULL,"Toolbar.bmp",IMAGE_BITMAP,0,0,LR_LOADFROMFILE
		mov [hBmp],eax
		// Find the height of the images, we'll use this to initialize the image list
		invoke GetObject,[hBmp],SIZEOF BITMAP, offset bmpinfo

		// Initialize the imagelist
		CoInvoke(pImagelist,IImageList2.Initialize,[bmpinfo.bmHeight],[bmpinfo.bmHeight],ILC_COLOR32 + ILC_MASK,1,32)

		// Add the images in the to the image list
		CoInvoke(pImagelist,IImageList2.AddMasked ,[hBmp], 0, offset returndata)

		// The bitmap is copied into the image list so destroy the original
		invoke DeleteObject,[hBmp]

		// Need a second copy of the image list ? Couldn't be any quicker than this
		// All of the images will be copied to the new clone
		CoInvoke(pImagelist,IImageList2.Clone ,offset IID_IImageList2,offset pImageListLarge)

		// One of the new functionalities in IImageList2 is the ability to resize your image list
		// There is no equivalent to this in the flat api
		CoInvoke(pImageListLarge,IImageList2.Resize ,32,32)

		// Set the background color
		CoInvoke(pImagelist,IImageList2.SetBkColor,CLR_NONE,offset returndata)

		// Initialize our toolbar

		// Get the toolbar handle
		invoke GetDlgItem,[hwnd],1002
		mov [hToolBar],eax

		// First tell our toolbar that we expect it to use version 6
		// This should already be set but it never hurts to make sure
		invoke SendMessage,[hToolBar],CCM_SETVERSION,6,0

		// The pointer to the imagelist object is the handle to the image list it creates
		// We're only using one image list so set iImageID to 0
		invoke SendMessage,[hToolBar],TB_SETIMAGELIST,0,[pImagelist]

		// Indent the buttons by 8 pixels
		invoke SendMessage,[hToolBar],TB_SETINDENT,8,0
		// Add the toolbar buttons
		invoke SendMessage,[hToolBar],TB_BUTTONSTRUCTSIZE,sizeof TBBUTTON,0
		invoke SendMessage,[hToolBar],TB_ADDBUTTONS,nToolbarBtns,OFFSET tbrbtns
		// Reset the toolbar size
		invoke SendMessage,[hToolBar],TB_AUTOSIZE,0,0

		jmp >.EXIT

	.WM_COMMAND
	cmp D[uMsg],WM_COMMAND
	jne >.WM_CLOSE

		jmp >.EXIT

	.WM_CLOSE
	cmp D[uMsg],WM_CLOSE
	jne >.DEFPROC
		INVOKE EndDialog,[hwnd],0

	.DEFPROC
		mov eax,FALSE
		ret

	.EXIT

	mov eax, TRUE
	ret
ENDF

